﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Management;

namespace BREPipelineFramework.Helpers
{
    public class StaticHelpers
    {
        /// <summary>
        ///  Looks in the registry to determine the database connection string for the
        ///  BizTalk Management Database for the BizTalk Group that this machine belongs
        ///  to.
        /// </summary>
        /// <returns>Database connection string to BTS Management Database</returns>
        public static string GetMgmtDBConnectionString()
        {
            string BTSMgmtDBName = String.Empty, BTSMgmtDBServerName = String.Empty;
            string MgmtDBConnString = String.Empty;
            object syncRoot = new object();

            //check to see if the string has already been retrieved.  If not, get it and store it.
            if (MgmtDBConnString.Length == 0)
            {
                lock (syncRoot)
                {
                    if (MgmtDBConnString.Length == 0)
                    {
                        ManagementObjectSearcher searcher =
                            new ManagementObjectSearcher(@"root\MicrosoftBizTalkServer", "SELECT * FROM MSBTS_GroupSetting");
                        foreach (ManagementObject Group in searcher.Get())
                        {
                            if (Group != null)
                            {
                                Group.Get();
                                BTSMgmtDBName = Group["MgmtDbName"].ToString();
                                BTSMgmtDBServerName = Group["MgmtDbServerName"].ToString();

                            }
                        }

                        if (BTSMgmtDBName.Length == 0 || BTSMgmtDBServerName.Length == 0)
                            throw new ApplicationException("Unable to find Management Database Name or Management Database Server Name");

                        // Assuming Integrated Security is being used for database connection.
                        MgmtDBConnString = string.Format("SERVER={0};DATABASE={1};Integrated Security=SSPI", BTSMgmtDBServerName, BTSMgmtDBName);
                    }
                }
            }
            return MgmtDBConnString;
        }
    }
}
