using System;
using System.IO;
using System.Text;
using System.Drawing;
using System.Resources;
using System.Reflection;
using System.Diagnostics;
using System.Collections;
using System.ComponentModel;
using Microsoft.BizTalk.Message.Interop;
using Microsoft.BizTalk.Component.Interop;
using Microsoft.BizTalk.Component;
using Microsoft.BizTalk.Messaging;
using Microsoft.RuleEngine;
using Microsoft.BizTalk.Streaming;
using System.Globalization;
using Microsoft.XLANGs.BaseTypes;
using BTS;

namespace BREPipelineFramework.PipelineComponents
{
    [ComponentCategory(CategoryTypes.CATID_PipelineComponent)]
    [System.Runtime.InteropServices.Guid("8559e874-8877-4687-8e20-1176f6ffee02")]
    [ComponentCategory(CategoryTypes.CATID_Any)]
    public class BREPipelineFrameworkComponent : Microsoft.BizTalk.Component.Interop.IComponent, IBaseComponent, IPersistPropertyBag, IComponentUI
    {
        #region Private Properties

        private System.Resources.ResourceManager resourceManager = new System.Resources.ResourceManager("BREPipelineFramework.PipelineComponents.BREPipelineFrameworkComponent", Assembly.GetExecutingAssembly());
        private bool _Enabled = true;
        private string _ApplicationContext;
        private string _ExecutionPolicy;
        private string _InstructionLoaderPolicy;
        private bool recoverableInterchangeProcessingEnabled;

        #endregion

        #region Public Properties
        
        /// <summary>
        /// Whether this pipeline component is enabled or not, if set to false then the pipeline component will behave in a pass through manner
        /// </summary>
        public bool Enabled
        {
            get
            {
                return _Enabled;
            }
            set
            {
                _Enabled = value;
            }
        }
        
        /// <summary>
        /// The ApplicationContext which should be asserted as a string to the BRE Policy so that it can be used to selectively fire specific rules in the policy
        /// </summary>
        public string ApplicationContext
        {
            get
            {
                return _ApplicationContext;
            }
            set
            {
                _ApplicationContext = value;
            }
        }
        
        /// <summary>
        /// The name of the ExecutionPolicy to be executed
        /// </summary>
        public string ExecutionPolicy
        {
            get
            {
                return _ExecutionPolicy;
            }
            set
            {
                _ExecutionPolicy = value;
            }
        }
        
        /// <summary>
        /// The name of the InstructionLoaderPolicy to be executed
        /// </summary>
        public string InstructionLoaderPolicy
        {
            get
            {
                return _InstructionLoaderPolicy;
            }
            set
            {
                _InstructionLoaderPolicy = value;
            }
        }

        /// <summary>
        /// Whether or not Recoverable Interchange Processing is enabled
        /// </summary>
        public bool RecoverableInterchangeProcessingEnabled
        {
            get { return recoverableInterchangeProcessingEnabled; }
            set { recoverableInterchangeProcessingEnabled = value; }
        }

        #endregion

        #region IBaseComponent members
        /// <summary>
        /// Name of the component
        /// </summary>
        [Browsable(false)]
        public string Name
        {
            get
            {
                return resourceManager.GetString("COMPONENTNAME", System.Globalization.CultureInfo.InvariantCulture);
            }
        }
        
        /// <summary>
        /// Version of the component
        /// </summary>
        [Browsable(false)]
        public string Version
        {
            get
            {
                return resourceManager.GetString("COMPONENTVERSION", System.Globalization.CultureInfo.InvariantCulture);
            }
        }
        
        /// <summary>
        /// Description of the component
        /// </summary>
        [Browsable(false)]
        public string Description
        {
            get
            {
                return resourceManager.GetString("COMPONENTDESCRIPTION", System.Globalization.CultureInfo.InvariantCulture);
            }
        }
        #endregion
        
        #region IPersistPropertyBag members
        /// <summary>
        /// Gets class ID of component for usage from unmanaged code.
        /// </summary>
        /// <param name="classid">
        /// Class ID of the component
        /// </param>
        public void GetClassID(out System.Guid classid)
        {
            classid = new System.Guid("8559e874-8877-4687-8e20-1176f6ffee02");
        }
        
        /// <summary>
        /// not implemented
        /// </summary>
        public void InitNew()
        {
        }
        
        /// <summary>
        /// Loads configuration properties for the component
        /// </summary>
        /// <param name="pb">Configuration property bag</param>
        /// <param name="errlog">Error status</param>
        public virtual void Load(Microsoft.BizTalk.Component.Interop.IPropertyBag pb, int errlog)
        {
            object val = null;
            val = this.ReadPropertyBag(pb, "Enabled");
            if ((val != null))
            {
                this._Enabled = ((bool)(val));
            }
            val = this.ReadPropertyBag(pb, "ApplicationContext");
            if ((val != null))
            {
                this._ApplicationContext = ((string)(val));
            }
            val = this.ReadPropertyBag(pb, "ExecutionPolicy");
            if ((val != null))
            {
                this._ExecutionPolicy = ((string)(val));
            }
            val = this.ReadPropertyBag(pb, "InstructionLoaderPolicy");
            if ((val != null))
            {
                this._InstructionLoaderPolicy = ((string)(val));
            }
            val = this.ReadPropertyBag(pb, "RecoverableInterchangeProcessingEnabled");
            if ((val != null))
            {
                this.recoverableInterchangeProcessingEnabled = ((bool)(val));
            }
        }
        
        /// <summary>
        /// Saves the current component configuration into the property bag
        /// </summary>
        /// <param name="pb">Configuration property bag</param>
        /// <param name="fClearDirty">not used</param>
        /// <param name="fSaveAllProperties">not used</param>
        public virtual void Save(Microsoft.BizTalk.Component.Interop.IPropertyBag pb, bool fClearDirty, bool fSaveAllProperties)
        {
            this.WritePropertyBag(pb, "Enabled", this.Enabled);
            this.WritePropertyBag(pb, "ApplicationContext", this.ApplicationContext);
            this.WritePropertyBag(pb, "ExecutionPolicy", this.ExecutionPolicy);
            this.WritePropertyBag(pb, "InstructionLoaderPolicy", this.InstructionLoaderPolicy);
            this.WritePropertyBag(pb, "RecoverableInterchangeProcessingEnabled", this.recoverableInterchangeProcessingEnabled);
        }
        
        #region utility functionality
        /// <summary>
        /// Reads property value from property bag
        /// </summary>
        /// <param name="pb">Property bag</param>
        /// <param name="propName">Name of property</param>
        /// <returns>Value of the property</returns>
        private object ReadPropertyBag(Microsoft.BizTalk.Component.Interop.IPropertyBag pb, string propName)
        {
            object val = null;
            try
            {
                pb.Read(propName, out val, 0);
            }
            catch (System.ArgumentException )
            {
                return val;
            }
            catch (System.Exception e)
            {
                throw new System.ApplicationException(e.Message);
            }
            return val;
        }
        
        /// <summary>
        /// Writes property values into a property bag.
        /// </summary>
        /// <param name="pb">Property bag.</param>
        /// <param name="propName">Name of property.</param>
        /// <param name="val">Value of property.</param>
        private void WritePropertyBag(Microsoft.BizTalk.Component.Interop.IPropertyBag pb, string propName, object val)
        {
            try
            {
                pb.Write(propName, ref val);
            }
            catch (System.Exception e)
            {
                throw new System.ApplicationException(e.Message);
            }
        }
        #endregion
        #endregion
        
        #region IComponentUI members
        /// <summary>
        /// Component icon to use in BizTalk Editor
        /// </summary>
        [Browsable(false)]
        public IntPtr Icon
        {
            get
            {
                return ((System.Drawing.Bitmap)(this.resourceManager.GetObject("COMPONENTICON", System.Globalization.CultureInfo.InvariantCulture))).GetHicon();
            }
        }
        
        /// <summary>
        /// The Validate method is called by the BizTalk Editor during the build 
        /// of a BizTalk project.
        /// </summary>
        /// <param name="obj">An Object containing the configuration properties.</param>
        /// <returns>The IEnumerator enables the caller to enumerate through a collection of strings containing error messages. These error messages appear as compiler error messages. To report successful property validation, the method should return an empty enumerator.</returns>
        public System.Collections.IEnumerator Validate(object obj)
        {
            // example implementation:
            // ArrayList errorList = new ArrayList();
            // errorList.Add("This is a compiler error");
            // return errorList.GetEnumerator();
            return null;
        }
        #endregion
        
        #region IComponent members
        /// <summary>
        /// Implements IComponent.Execute method.
        /// </summary>
        /// <param name="pc">Pipeline context</param>
        /// <param name="inmsg">Input message</param>
        /// <returns>Original input message</returns>
        /// <remarks>
        /// IComponent.Execute method is used to initiate
        /// the processing of the message in this pipeline component.
        /// </remarks>
        public Microsoft.BizTalk.Message.Interop.IBaseMessage Execute(Microsoft.BizTalk.Component.Interop.IPipelineContext pc, Microsoft.BizTalk.Message.Interop.IBaseMessage inmsg)
        {
            if (_Enabled)
            {
                BREPipelineMetaInstructionCollection _BREPipelineMetaInstructionCollection = new BREPipelineMetaInstructionCollection();
                _BREPipelineMetaInstructionCollection.Pc = pc;

                //Get a stream containing the body part of the original message
                IBaseMessagePart msgPart = inmsg.BodyPart;
                Stream msgStream = msgPart.GetOriginalDataStream();

                //Copy the original stream to a new stream so that the original message is preserved
                VirtualStream newMsgStream = new VirtualStream();
                msgStream.CopyTo(newMsgStream);
                newMsgStream.Seek(0, SeekOrigin.Begin);

                //Create a copy of the original body part and copy over it's properties as well
                IBaseMessagePart copiedBodyPart = pc.GetMessageFactory().CreateMessagePart();
                copiedBodyPart.Data = newMsgStream;
                copiedBodyPart.PartProperties = inmsg.BodyPart.PartProperties;

                //Create a new message with a body part in the context manipulator collection and set the body part to the copied message stream
                _BREPipelineMetaInstructionCollection.InMsg = pc.GetMessageFactory().CreateMessage();
                CopyMessageParts(inmsg, _BREPipelineMetaInstructionCollection.InMsg, copiedBodyPart);

                //Add the new message body part's stream to the Pipeline Context's resource tracker so that it will be disposed off correctly
                pc.ResourceTracker.AddResource(newMsgStream);

                //Create and copy over context from the original message onto the message in the context manipulator collection
                _BREPipelineMetaInstructionCollection.InMsg.Context = pc.GetMessageFactory().CreateMessageContext();

                // Iterate through inbound message context properties and add to the new outbound message
                for (int contextCounter = 0; contextCounter < inmsg.Context.CountProperties; contextCounter++)
                {
                    string Name;
                    string Namespace;

                    object PropertyValue = inmsg.Context.ReadAt(contextCounter, out Name, out Namespace);

                    // If the property has been promoted, respect the settings
                    if (inmsg.Context.IsPromoted(Name, Namespace))
                    {
                        _BREPipelineMetaInstructionCollection.InMsg.Context.Promote(Name, Namespace, PropertyValue);
                    }
                    else
                    {
                        _BREPipelineMetaInstructionCollection.InMsg.Context.Write(Name, Namespace, PropertyValue);
                    }
                }

                try
                {
                    if (!string.IsNullOrEmpty(_InstructionLoaderPolicy))
                    {
                        //Execute the InstructionLoaderPolicy to instantiate the relevant MetaInstructions
                        using (Microsoft.RuleEngine.Policy policy = new Policy(_InstructionLoaderPolicy))
                        {
                            object[] instructionLoaderFacts = { _ApplicationContext, _BREPipelineMetaInstructionCollection };
                            policy.Execute(instructionLoaderFacts);
                        }
                    }

                    BREPipelineFramework.SampleInstructions.MetaInstructions.ContextMetaInstructions contextInstruction = new SampleInstructions.MetaInstructions.ContextMetaInstructions();
                    BREPipelineFramework.SampleInstructions.MetaInstructions.HelperMetaInstructions helperInstruction = new SampleInstructions.MetaInstructions.HelperMetaInstructions();

                    _BREPipelineMetaInstructionCollection.AddMetaInstruction(contextInstruction);
                    _BREPipelineMetaInstructionCollection.AddMetaInstruction(helperInstruction);

                    //Execute the ExecutionPolicy using the instantiated MetaInstructions as facts
                    if (!string.IsNullOrEmpty(_ExecutionPolicy))
                    {
                        using (Microsoft.RuleEngine.Policy policy = new Policy(_ExecutionPolicy))
                        {
                            object[] pipelineMetaInstructionFacts = new object[_BREPipelineMetaInstructionCollection.GetCount() + 2];
                            pipelineMetaInstructionFacts[0] = _ApplicationContext;
                            pipelineMetaInstructionFacts[1] = _BREPipelineMetaInstructionCollection.Pc;

                            for (int i = 1; i <= _BREPipelineMetaInstructionCollection.GetCount(); i++)
                            {
                                pipelineMetaInstructionFacts[i + 1] = _BREPipelineMetaInstructionCollection.GetMetaInstructionByIndex(i - 1);
                            }

                            policy.Execute(pipelineMetaInstructionFacts);

                            //If any of the MetaInstructions have returned exceptions then throw them now.
                            _BREPipelineMetaInstructionCollection.ThrowExceptions();

                            //Execute all the instructions that have been loaded into the MetaInstructions by the ExecutionPolicy
                            _BREPipelineMetaInstructionCollection.Execute();
                        }
                    }
                }
                catch (Exception e)
                {
                    if (recoverableInterchangeProcessingEnabled)
                    {
                        _BREPipelineMetaInstructionCollection.InMsg.SetErrorInfo(e);
                        PropertyBase messageDestinationProperty = new MessageDestination();
                        _BREPipelineMetaInstructionCollection.InMsg.Context.Write(messageDestinationProperty.QName.Name, messageDestinationProperty.QName.Namespace, "SuspendQueue");
                    }
                    else
                    {
                        throw;
                    }
                }

                if (recoverableInterchangeProcessingEnabled)
                {
                    PropertyBase suspendOnRoutingFailureProperty = new SuspendMessageOnRoutingFailure();
                    _BREPipelineMetaInstructionCollection.InMsg.Context.Write(suspendOnRoutingFailureProperty.QName.Name, suspendOnRoutingFailureProperty.QName.Namespace, true);
                }

                //Return the updated message
                return _BREPipelineMetaInstructionCollection.InMsg;
            }

            //Return the original message, treating this pipeline component as if it were a pass through pipeline component
            return inmsg;
        }

        /// <summary>
        /// Copy over all parts of the message from a source message to a destination message, choosing which part should be the body part
        /// </summary>
        /// <param name="sourceMessage"></param>
        /// <param name="destinationMessage"></param>
        /// <param name="newBodyPart"></param>
        private void CopyMessageParts(IBaseMessage sourceMessage, IBaseMessage destinationMessage, IBaseMessagePart newBodyPart)
        {
            string bodyPartName = sourceMessage.BodyPartName;
            for (int c = 0; c < sourceMessage.PartCount; ++c)
            {
                string partName = null;
                IBaseMessagePart messagePart = sourceMessage.GetPartByIndex(c, out partName);
                if (partName != bodyPartName)
                {
                    destinationMessage.AddPart(partName, messagePart, false);
                }
                else
                {
                    destinationMessage.AddPart(bodyPartName, newBodyPart, true);
                }
            }
        }

        private static void HandleRecoverableError(IBaseMessage outMsg, Exception e)
        {
        }

        private static void HandleNonRecoverableError()
        {

        }

        #endregion
    }
}
